  /*----------------------------------------------------------------------------
 Copyright:      W.Wallucks
 Remarks:        
 known Problems: none
 Version:        04.05.2008
 Description:    messen und regeln mit dem Webmodul

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

/**
 * \addtogroup messen Datenaufnahme und Logging
 *
 * @{
  */

/**
 * \file
 * Messung.h
 * Definitionen der Ports fr angeschlossene Relais und Schalter
 *
 * \author W.Wallucks
 */

/**
 * @}
 */
#ifndef _MESSUNG_H_
	#define _MESSUNG_H_

/**
 * \port
 *	- Port C - An/Aus Sensoren an Pin 0, 1, 6 und 7
 *	- Portausgang mit Schalter an PD4, PD5 und PD6
 */
#define SENS_Read			PINA	//!< Porteingang an dem die An/Aus Sensoren angeschlossen sind
#define SENS_PULLUP			PORTA	//
#define SENS_DDR			DDRA	
#define SENS_INTENA			PCIE0	// == Port A PortChangeInteruptEnable0
#define SENS_INTMASK		PCMSK0
#define SENS_PIN1			4		//!< Sensor an Pin 0
#define SENS_PIN2			5		//!< Sensor an Pin 1
#define SENS_PIN3			6		//!< Sensor an Pin 2
#define SENS_PIN4			7		//!< Sensor an Pin 3
#define SENS_ACTIVE_PINS	((1<<SENS_PIN1) | (1<<SENS_PIN2) | (1<<SENS_PIN3) | (1<<SENS_PIN4))



#define PORT_SCHALTER1		PC2		//!< Portausgang mit Schalter PC2
#define PORT_SCHALTER2		PC3		//!< Portausgang mit Schalter PC3	
#define PORT_SCHALTER3		PC4		//!< Portausgang mit Schalter PC4
#define PORT_SCHALTER4		PC5		//!< Portausgang mit Schalter PC5
#define PORT_SCHALTER5		PC6		//!< Portausgang mit Schalter PC6
#define PORT_SCHALTER6		PC7		//!< Portausgang mit Schalter PC7
#define OUT_DDR				DDRC
#define OUT_PORT			PORTC


/*
* Schalter1-4 schalten
*/
#define S1An()		OUT_PORT |= (1<<PORT_SCHALTER1)		//!< Port high schalten
#define S1Aus()		OUT_PORT &= ~(1<<PORT_SCHALTER1)	//!< Port low  schalten
#define S1Toggle()	OUT_PORT ^= (1<<PORT_SCHALTER1)		//!< Port umschalten
#define S2An()		OUT_PORT |= (1<<PORT_SCHALTER2)
#define S2Aus()		OUT_PORT &= ~(1<<PORT_SCHALTER2)
#define S2Toggle()	OUT_PORT ^= (1<<PORT_SCHALTER2)
#define S3An()		OUT_PORT |= (1<<PORT_SCHALTER3)
#define S3Aus()		OUT_PORT &= ~(1<<PORT_SCHALTER3)
#define S3Toggle()	OUT_PORT ^= (1<<PORT_SCHALTER3)
#define S4An()		OUT_PORT |= (1<<PORT_SCHALTER4)
#define S4Aus()		OUT_PORT &= ~(1<<PORT_SCHALTER4)
#define S4Toggle()	OUT_PORT ^= (1<<PORT_SCHALTER4)
#define S5An()		OUT_PORT |= (1<<PORT_SCHALTER5)
#define S5Aus()		OUT_PORT &= ~(1<<PORT_SCHALTER5)
#define S5Toggle()	OUT_PORT ^= (1<<PORT_SCHALTER5)
#define S6An()		OUT_PORT |= (1<<PORT_SCHALTER6)
#define S6Aus()		OUT_PORT &= ~(1<<PORT_SCHALTER6)
#define S6Toggle()	OUT_PORT ^= (1<<PORT_SCHALTER6)



#if USE_OW
extern volatile int16_t ow_array[MAXSENSORS];	// Speicherplatz fr 1-wire Sensorwerte
#endif

//----------------------------------------------------------------------------
// Prototypen
void messung_init(void);
uint8_t start_OW(void);
void lese_Temperatur(void);
void set_SensorResolution(void);
int16_t initSchaltzeiten(char *);
void regelAnlage(SOLL_STATUS *);
void log_init(void);
void log_status(void);

		
#endif
